# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from . import decorators, properties
from .page import OGSubpage
from .profile import OGProfile


@decorators.OGClass('article', ('page', 'section', 'published_time', 'modified_time', 'expiration_time', 'author!s', 'tag!s'))
class OGArticlePage(OGSubpage):
    """
    Namespace URI: https://ogp.me/ns/article#
    _________________________________________

    Entities:

    article:published_time  -> When the article was first published.
    article:modified_time   -> When the article was last changed.
    article:expiration_time -> When the article is out of date after.
    article:section         -> A high-level section name. E.g. Technology
    article:author          -> Writers of the article.
    article:tag             -> Tag words associated with this article.

    _________________________________________
    """

    class Meta:
        db_table = 'articles'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('article meta information')
        verbose_name_plural = _('articles meta information')

    section = models.CharField(_('section'), max_length=127, null=True)
    published_time = models.DateTimeField(_('date of the publication'), null=True)
    modified_time = models.DateTimeField(_('date of the modification'), null=True)
    expiration_time = models.DateTimeField(_('date of the expiration'), null=True)
    authors = models.ManyToManyField(OGProfile, null=True, verbose_name=_('authors'))
    tags = models.ManyToManyField(properties.OGTag, null=True, verbose_name=_('tags'))

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith('_time'):
            v = v.strftime("%Y-%m-%dT%H:%M")
        yield k, v

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'article'
        super().__init__(*args, **kwargs)
