# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from . import decorators, properties
from .page import OGSubpage
from .profile import OGProfile


@decorators.OGClass('book', ('page', 'isbn', 'release_date', 'author!s', 'tag!s'))
class OGBookPage(OGSubpage):
    """
    Namespace URI: https://ogp.me/ns/book#
    _________________________________________

    Entities:

    book:release_date -> The date the book was released.
    book:isbn         -> International Standard Book Number. ISBNs consist of 13 digits.
    book:author       -> Writers of the book.
    book:tag          -> Tag words associated with this book.

    _________________________________________
    """

    class Meta:
        db_table = 'books'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('book meta information')
        verbose_name_plural = _('books meta information')

    isbn = models.CharField(_('isbn'), max_length=13, null=True)
    release_date = models.DateField(_('date of the release'), null=True)
    authors = models.ManyToManyField(OGProfile, null=True, verbose_name=_('authors'))
    tags = models.ManyToManyField(properties.OGTag, null=True, verbose_name=_('tags'))

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'book'
        super().__init__(*args, **kwargs)
