# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.db import models
from django.conf import settings
from django.utils.translation import gettext_lazy as _

from ... import decorators
from .currencies import Currencies


@decorators.OGClass(':price', ('amount', 'currency'))
class OGPrice(models.Model):
    """
    URI: https://developers.facebook.com/docs/marketing-api/catalog/reference/#og-tags
    _________________________________________

    Entities:

    ...:price:amount      -> Discounted price if the item is on sale. Use "." as the decimal for the sale price.
                             The sale price is required if you plan to use an overlay for discounted prices
    ...:price:currency    -> Currency of the discounted price if the item is on sale in3-digit ISO currency code
    ...:price_dates:start -> [Extra] Start date and time for your sale in your timezone, written as
                               YYYY-MM-DDT0:00-23:59/YYYY-MM-DDT0:00-23:59, separated by a slash.
                             * Write the start and end dates as YYYY-MM-DD.
                             * Write the time in a 24-hour format (0:00 to 23:59).
                             * Add a "T" after each date and then include the time,
                               where the end time represents the time zone.
                             In the example, 03:00 denotes the time zone
    ...:price_dates:end   -> [Extra] End date and time for your sale in your timezone, written as
                             YYYY-MM-DDT0:00-23:59/YYYY-MM-DDT0:00-23:59, separated by a slash.
                             * Write the start and end dates as YYYY-MM-DD.
                             * Write the time in a 24-hour format (0:00 to 23:59).
                             * Add a "T" after each date and then include the time,
                               where the end time represents the time zone.
                             In the example, 03:00 denotes the time zone

    _________________________________________
    """

    Currencies = Currencies

    class Meta:
        db_table = 'prices'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('price information')
        verbose_name_plural = _('prices information')

    amount = models.DecimalField(_('price amount'), max_digits=10, decimal_places=2)
    currency = models.PositiveSmallIntegerField(_('price currency'), choices=Currencies.choices)

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith(':currency'):
            v = cls.Currencies.find_currency(v)
        yield k, str(v)
