import json
from pathlib import Path
from typing import Generator, Tuple
from django.conf import settings
from django.utils.translation import gettext_noop as _n

fpath = Path(__file__).resolve().parent.parent.parent.parent / 'data' / 'taxonomy.json'
prefix = "OPENGRAPH_PRODUCT_TAXONOMY"
depth = settings.get(f"{prefix}_DEPTH", 2)
base = settings.get(f"{prefix}_BASE", None)
_fullname = settings.get(f"{prefix}_FULLNAME", False)

with open(settings.get(f"{prefix}_FILEPATH", fpath), 'r', -1, 'utf8') as file:
    data = json.load(file)

del prefix, fpath

if base:
    prefix = ''
    ctx = data
    for i in base.split('.'):
        if i not in ctx['values'].keys():
            raise ValueError(f'scope {i} not found in the {base}')
        if 'name' in ctx.keys():
            prefix += f'> {ctx["name"]}' if prefix else ctx["name"]
        ctx = ctx['values'][i]
    ctx['name'] = f'{prefix} > {ctx["name"]}'
    data = ctx

    del base, prefix, ctx, i

if depth:
    def clean(obj, d=0):
        if d < depth:
            for k in obj["values"].keys():
                clean(obj["values"][k], d + 1)
        else:
            obj["values"] = {}

    clean(data)
    del clean


class TaxonomyMeta(type):

    def __new__(mcs, name, bases, attrs):
        def generate(obj:dict, prev=None):
            cur = Taxonomy(obj.get('id', 0), obj.get('name', ''), prev)
            for k, v in obj.get('values', {}).items():
                setattr(cur, k, generate(v, prev=cur))
            return cur

        meta = attrs.pop('_meta', None)
        cls = super().__new__(mcs, name, bases, attrs)

        if meta:
            ctx = generate(meta)
            cls = cls(ctx._id, ctx._name, ctx._prev)
            for k, v in ctx.__dict__.items():
                setattr(cls, k, v)
        return cls


class Taxonomy(metaclass=TaxonomyMeta):
    @property
    def _choices(self):
        if self.id:
            yield self.id, self.name if not _fullname else self.fullname
        for k in self.categories():
            yield from getattr(self, k)._choices

    @property
    def name(self) -> str:
        if self._name.count('>'):
            return self._name.split('>')[-1].strip()
        else:
            return self._name

    @property
    def id(self) -> int:
        return self._id

    @property
    def fullname(self) -> str:
        return f'{self._prev.fullname} > {self.name}' if self._prev and self._prev.fullname else self.name

    @property
    def choices(self) -> Tuple[Tuple[int, str], ...]:
        return tuple([(k, _n(v)) for k, v in self._choices])

    def categories(self) -> Generator:
        keys = list(self.__dict__.keys())
        keys.sort()
        for k in keys:
            if not k.startswith('_') and isinstance(self.__dict__[k], Taxonomy):
                yield k

    def __getitem__(self, key: str) -> 'Taxonomy':
        obj = self
        ctx = key.split('.')
        k = None
        while 1:
            for k in obj.categories():
                if ctx[0] == k:
                    obj = getattr(obj, k)
                    ctx = ctx[1:]
                    break
                k = None
            if k is None:
                raise KeyError(key)
            if not ctx:
                return obj

    def __int__(self) -> int:
        return self.id

    def __bool__(self) -> bool:
        return self.id > 0

    def __str__(self) -> str:
        return f'{self.id} - {self.fullname}' if self else ''

    def __call__(self, id=0, name='', prev=None):
        self._prev = prev or self._prev
        self._id = id or self._id
        self._name = name or self._name
        return self

    def __init__(self, id=0, name='', prev=None):
        self._id = id
        self._name = name
        self._prev = prev if not prev or prev.id else None


class Categories(Taxonomy):
    _meta = data

    class NotFound(ValueError):
        pass

    def find_category(self, id: int) -> Taxonomy:
        def find(obj, id):
            if obj.id == id:
                return obj
            for i in obj.categories():
                found = find(obj[i], id)
                if found is not None:
                    return found
            return None

        found = find(self, id)
        if not found:
            raise self.NotFound(f'category id {id}')
        return found


del data
