# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from typing import Iterable, Any

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.utils.translation import get_language, to_locale
from django.db import models

from . import decorators
from .properties import OGImage


@decorators.OGClass('og', ('url', 'type', 'title', 'description', 'determiner', 'locale', 'locales_alt', 'site_name', 'image!s'))
class OGWebPage(models.Model):
    """
    Namespace URI: https://ogp.me/ns/website#
    _________________________________________

    Entities:

    og:url              -> The canonical URL of your object that will be used as its permanent ID in the graph.
    og:type             -> The type of this object.
    og:title            -> The title of your object as it should appear within the graph, e.g., "The Rock".
    og:determiner       -> The word that appears before this object's title in a sentence. An enum of (a, an, the, "", auto).
    og:description      -> A one to two sentence description of your object.
    og:locale           -> The locale these tags are marked up in. Of the format language_TERRITORY. Default is en_US.
    og:locale:alternate -> An array of other locales this page is available in.
    og:site_name        -> If your object is part of a larger website, the name which should be displayed for the overall site. e.g., "IMDb".
    og:image            -> An image URL which should represent your object within the graph.
    og:video            -> A URL to a video file that complements this object.
    og:audio            -> A URL to an audio file to accompany this object.

    _________________________________________
    """

    class Meta:
        db_table = 'pages'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('page meta information')
        verbose_name_plural = _('pages meta information')

    url = models.URLField(_('url'), max_length=255, unique=True)
    type = models.CharField(_('open graph type'), max_length=32, default='webpage')
    title = models.CharField(_('title'), max_length=127, null=True)
    description = models.CharField(_('description'), max_length=255, null=True)
    images = models.ManyToManyField(OGImage, null=True, on_delete=models.SET_NULL, verbose_name=_('images'))

    @property
    def page(self):
        return self

    @property
    def locale(self) -> str | None:
        return to_locale(get_language())

    @property
    def locales_alt(self) -> Iterable[str] | None:
        """has not the default realization"""
        return None

    @property
    def determiner(self) -> str | None:
        """has not the default realization"""
        return None

    @property
    def site_name(self) -> str | None:
        return settings.get('OPENGRAPH_SITE_NAME', None)

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith(':locales_alt'):
            for x in v.locales_alt:
                yield f'{k[:-11]}:locale:alternate', x
        else: yield k, v

    def get_absolute_url(self):
        return self.url

    def __str__(self):
        return self.url


@decorators.ProxyProperties('page', ('url', 'title'))
class OGSubpage(models.Model):

    class Meta:
        abstract = True

    page = models.ForeignKey(OGWebPage, on_delete=models.CASCADE, unique=True)

    def get_absolute_url(self):
        return self.url

    def __str__(self):
        return self.url

    def __init__(self, *args,
                 page: OGWebPage | None = None,  # <- has priority
                 page_url: str | None = None,    # existed page would be found
                 page_type: str | None = None,   # in else page will be created
                 page_title: str | None = None,
                 page_description: str | None = None,
                 page_images: Any | None = None,
                 **kwargs):

        vargs = locals()
        [kwargs.update({k: vargs[k]}) for k in filter(lambda k: not k.endswith(('args', 'self')), vargs)]
        pg_args = {k[5:]: kwargs.pop(k) for k in (kwargs.keys()) if k.startswith('page_')}

        if not kwargs['page']:
            try:
                page = OGWebPage.objects.get(url=pg_args['url'])
            except OGWebPage.DoesNotExist:
                page = OGWebPage.objects.create(**pg_args)
            kwargs['page'] = page

        if pg_args['type'] and page.type.lower() != pg_args['type'].lower():
            raise TypeError(f"{self.__class__.__name__}: Expected OpenGraph type of {page.__name__} "
                            f"is '{pg_args['type']}', but was given '{page.type}'")

        super().__init__(*args, **kwargs)
