# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.db import models

from .. import decorators
from ..profile import OGProfile

try:
    from django.conf import settings
    from django.utils.translation import gettext_lazy as _
except ImportError:
    settings = {}
    _ = lambda x: x


@decorators.OGClass(':actor', ('!url', 'role'))
@decorators.ProxyProperties('actor', ('title', 'url', 'username', 'first_name', 'last_name', 'is_male', 'gender'))
class OGActor(models.Model):
    """
    Namespace URI: https://ogp.me/ns/profile#
    _________________________________________

    Entities:

    ...:actor[:url]      -> Link to the actor profile page.
    ...:actor:role       -> Their role

    _________________________________________
    """

    class Meta:
        db_table = 'actors'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('actor meta information')
        verbose_name_plural = _('actors meta information')

    actor = models.ForeignKey(OGProfile, on_delete=models.CASCADE, verbose_name=_('actor'))
    role = models.CharField(_('role'), max_length=127, null=True)

    def get_absolute_url(self):
        return self.url

    def __str__(self):
        return self.url

    def __init__(self, *args, **kwargs):
        pg_args = {k: kwargs.pop(k) for k in (kwargs.keys()) if k.startswith('page_')}
        pg_args.update({k[6:]: kwargs.pop(k) for k in (kwargs.keys()) if k.startswith('actor_')})
        kwargs['actor'] = kwargs.get('actor', False) or OGProfile(**pg_args)

        super().__init__(*args, **kwargs)
