# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.db import models

from .. import decorators

try:
    from django.conf import settings
    from django.utils.translation import gettext_lazy as _
except ImportError:
    settings = {}
    _ = lambda x: x


class OGMedia(models.Model):
    """
    Namespace URI: https://ogp.me/ns/profile#
    _________________________________________

    Entities:

    ...:media            -> The canonical URL of your object that will be used as its permanent ID in the graph.
    ...:media:type       -> The MIME type of this media object.
    ...:media:secure_url -> An alternate url to use if the webpage requires HTTPS.
    ...:media:width      -> The number of pixels wide.
    ...:media:height     -> The number of pixels height.
    ...:media:alt        -> A description of what is in the object

    _________________________________________
    """

    class Meta:
        db_table = 'media'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('media file meta information')
        verbose_name_plural = _('media files meta information')

    url = models.URLField(_('url'), max_length=255, unique=True)
    type = models.CharField(_('mime type'), max_length=64)
    secure_url = models.URLField(_('secure url'), max_length=255, null=True)
    width = models.PositiveSmallIntegerField(_('width'), null=True)
    height = models.PositiveSmallIntegerField(_('height'), null=True)
    alt = models.CharField(_('alternative text'), max_length=255, null=True)

    def get_absolute_url(self):
        return self.url

    @classmethod
    def create_image(cls, *args, **kwargs):
        return OGImage(*args, **kwargs)

    @classmethod
    def create_video(cls, *args, **kwargs):
        return OGVideoStatic(*args, **kwargs)

    @classmethod
    def create_audio(cls, *args, **kwargs):
        return OGAudioStatic(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        if self.__class__ not in (OGImage, OGVideoStatic, OGAudioStatic):
            raise AttributeError('object creation is not allowed, '
                                 f'use {__class__.__name__}.create_[image | video | audio] instead')
        super().__init__(*args, **kwargs)


@decorators.OGClass(':image', ('!url', 'type', 'secure_url', 'width', 'height', 'alt'))
class OGImage(OGMedia):

    class Meta:
        proxy = True


@decorators.OGClass('og:video', ('!url', 'type', 'secure_url', 'width', 'height'))
class OGVideoStatic(OGMedia):

    class Meta:
        proxy = True


@decorators.OGClass('og:audio', ('!url', 'type', 'secure_url'))
class OGAudioStatic(OGMedia):

    class Meta:
        proxy = True
